/* 
* A primeira seção da especificação vai até o primeiro %%,
* e consiste de código Java que é copiado ao pé da letra
*
*/

package lex;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.io.IOException;

%%

/*
* A segunda seção vai até o próximo %%, e são diversos parâmetros
* de configuração, além de código Java copiado para o corpo da
* classe do analisador léxico
*
*/

%class ScannerJF        // nome da classe do analisador
%public                 // classe deve ser pública
%line                   // guarde número da linha em yyline
%column                 // guarde número da coluna em yycolumn
%function nextToken     // nome do método que vai fornecer um token
%type Token             // classe usado para tokens
// interface implementada pelo analisador
%implements Scanner     

// Código Java entre %{ e %} é copiado pro corpo da classe
// do analisador
%{ 

	public ScannerJF() { }

	public void setInput(String input) {
	    // inicializa entrada pro analisador
		yyreset(new StringReader(input));
	}
	
	public List<Token> allTokens() throws IOException {
		List<Token> tokens = new ArrayList<Token>();
		Token tok = nextToken();
		while(tok.type != Token.EOF) {
			tokens.add(tok);
			tok = nextToken();
		}
		return tokens;
	}

%}


%%

/*
* A última seção contém as regras léxicas, cada regra é um
* par com uma expressão regular e um trecho de código Java
* entre { e }.
*
*/

// Espaços são ignorados
[ \r\n\t]    { }

[0-9]+       { return new Token(1, yytext()); }
[(]          { return new Token('(', yytext()); }
[)]          { return new Token(')', yytext()); }
[+]          { return new Token('+', yytext()); }
[-]          { return new Token('-', yytext()); }
[*]          { return new Token('*', yytext()); }
[/]          { return new Token('/', yytext()); }

// <<EOF>> é uma expressão para denotar final de entrada
<<EOF>>      { return new Token(Token.EOF, ""); }

// Erros léxicos devem ser capturados por essa regra ou são ignorados
.            { throw new RuntimeException("erro léxico, linha: " + 
               yyline + ", coluna : " + yycolumn + ", char: " + 
               yytext()); }

